/***************************************************************************
 *   Copyright (C) 2003 by Jens Dagerbo                                    *
 *   jens.dagerbo@swipnet.se                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __REPLACEITEM_H__
#define __REPLACEITEM_H__

#include <QTreeWidgetItem>

#include <kdeversion.h>
#include <kdebug.h>

#include "replaceview.h"

class ReplaceItem : public QTreeWidgetItem
{
public:
    // the file item
    ReplaceItem( ReplaceView * parent, ReplaceItem * after, QString file ) :
            QTreeWidgetItem( dynamic_cast<QTreeWidget *>(parent),
                            dynamic_cast<QTreeWidgetItem *>(after),
                            QTreeWidgetItem::Type ),
            _file( file ), _string( file ), _line( 0 ), _isfile( true ),
            _lineclicked( false ), _clicked( true )
    {
        setExpanded( true );
	setText( 0, file );
        setCheckState( 0, Qt::Checked ); // setOn( true );
    }

    // the line item
    ReplaceItem( ReplaceItem * parent, ReplaceItem * after, QString file, QString string, int line ) :
            QTreeWidgetItem( dynamic_cast<QTreeWidget *>(parent),
                            dynamic_cast<QTreeWidgetItem *>(after),
                            QTreeWidgetItem::Type ),
            _file( file ), _string( string ), _line( line ), _isfile( false ),
            _lineclicked( false ), _clicked( true )
    {
	setText( 0, QString::number( line + 1 ) + ": " + string );
        setCheckState( 0, Qt::Checked ); // setOn( true );
    }

    QString const & file() const
    {
        return _file;
    }

    int line() const
    {
        return _line;
    }

    QString const & string() const
    {
        return _string;
    }

    bool isFile() const
    {
        return _isfile;
    }

    bool justClicked()
    {
        bool t = _clicked;
        _clicked = true;
        return t;
    }

    bool lineClicked()
    {
        return _lineclicked;
    }

    ReplaceItem * parent() const
    {
        return static_cast<ReplaceItem*>( QTreeWidgetItem::parent() );
    }

    ReplaceItem * child( int index ) const
    {
        return static_cast<ReplaceItem*>( QTreeWidgetItem::child( index ) );
    }

    bool isOn() const
    {
        return checkState( 0 ) == Qt::Checked ? true : false;
    }

    void setOn( bool state )
    {
        if ( state )
            setCheckState( 0, Qt::Checked );
        else
            setCheckState( 0, Qt::Unchecked );
    }

//     ReplaceItem * firstChild() const
//     {
//         return static_cast<ReplaceItem*>( QTreeWidgetItem::child( _curchild ) );
//     }
// 
//     ReplaceItem * nextSibling() const
//     {
// 	if ( _curchild < ( childCount() - 1 ) ) {
// 	    _curchild++;
//             return static_cast<ReplaceItem*>( QTreeWidgetItem::child( _curchild ) );
// 	}
// 	else
// 	    return NULL;
//     }

/*    void activate( int column, QPoint const & localPos );*/
    bool hasCheckedChildren() const;
    virtual void stateChange( bool state );

    static bool s_listview_done;

private:
//     void paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align );
    void setChecked( bool checked );

    QString _file;
    QString _string;
    int _line;
    bool const _isfile;
    bool _lineclicked;
    bool _clicked;
};

#endif

